@@ -27,17 +27,30 @@
     #include <QMaemo5InformationBox>
 #endif
 
-
 #define APPNAME "QCPUFreq"
-#define APPVERSION "0.3.2"
+#define APPVERSION "0.3.3"
 
 MainWindow::MainWindow(QWidget *parent) :
     QMainWindow(parent),
-    ui(new Ui::MainWindow)
+    ui(new Ui::MainWindow),
+    //do not allow overclocking per default
+    allowOverclocking(false),
+    //create helper process
+    helperProcess( this ),
+    //create a new, stackable help window
+    helpWindow( this ),
+    //set minFreq to 0
+    minFreq(0),
+    //create UI refresh timer
+    refreshTimer( this ),
+    //create a QGraphicsScene for the little chip icon
+    scene( this ),
+    //show errors about the sudo setup only once
+    showSudoError( true )
 {
     //this is a stacked window on Maemo 5
     #if defined(Q_WS_MAEMO_5)
-	setAttribute(Qt::WA_Maemo5StackedWindow);
+        setAttribute(Qt::WA_Maemo5StackedWindow);
     #endif
 
     ui->setupUi(this);
@@ -47,27 +60,17 @@ MainWindow::MainWindow(QWidget *parent) :
     // enable auto rotation
     setAutoRotation();
 
-    //create a QGraphicsScene for the little chip icon
-    scene = new QGraphicsScene();
+    //initialize orientation
     orientationChanged();
 
-    //create the refresh timer
-    refreshTimer = new QTimer();
     //refresh UI every 10 seconds
-    refreshTimer->start( 10000 );
+    refreshTimer.start( 10000 );
 
-    //create helper process
-    helperProcess = new QProcess;
-
-    //create a new, stackable help window
-    helpWindow = new HelpWindow( this );
+    // initialize stackable help window
     #if defined(Q_WS_MAEMO_5)
-	helpWindow->setAttribute(Qt::WA_Maemo5StackedWindow);
+        helpWindow.setAttribute(Qt::WA_Maemo5StackedWindow);
     #endif
-    helpWindow->setWindowFlags( windowFlags() | Qt::Window );
-
-    //show errors about the sudo setup only once
-    showSudoError = true;
+    helpWindow.setWindowFlags( windowFlags() | Qt::Window );
 
     //connect signals and slots
     connect(ui->actionHelp, SIGNAL(triggered()), this, SLOT(showHelp()));
@@ -75,15 +78,18 @@ MainWindow::MainWindow(QWidget *parent) :
     connect( ui->freq_adjust, SIGNAL(valueChanged(int)), this, SLOT(adjustFreq()) );
     connect(QApplication::desktop(), SIGNAL(resized(int)), this, SLOT(orientationChanged()));
     connect(ui->sr_box, SIGNAL(clicked()), this, SLOT(setSmartReflex()));
-    connect(refreshTimer, SIGNAL(timeout()), this, SLOT(refresh()));
+    connect(&refreshTimer, SIGNAL(timeout()), this, SLOT(refresh()));
+    connect(ui->actionOverclocking, SIGNAL(toggled(bool)), this, SLOT(setOverclocking()));
+
+    //disable overclocking button on vanilla kernels
+    if ( getScalingFreq(0) <= 600000 ) {
+        ui->actionOverclocking->setDisabled(true);
+    }
 
 }
 
 MainWindow::~MainWindow()
 {
-    delete helpWindow;
-    delete refreshTimer;
-    delete scene;
     delete ui;
 }
 
@@ -108,10 +114,19 @@ void MainWindow::adjustFreq()
 
     //maxfreq should not be smaller than minfreq, because we do not want to decrease minfreq
     if (newmax < getMinFreq())
-	newmax = getMinFreq();
+        newmax = getMinFreq();
 
     max.setNum( newmax );
 
+    //check for overclocking
+    #if defined(Q_WS_MAEMO_5)
+    if (this->allowOverclocking == false && newmax > 600000) {
+        QMaemo5InformationBox::information(this, tr( "You need to enable overclocking in QCPUFreq's menu for setting frequencies above 600MHz!"), 0);
+        refresh();
+        return;
+    }
+    #endif
+
     callHelper( "set_maxfreq", max );
 
     refresh();
@@ -130,24 +145,24 @@ int MainWindow::callHelper(QString action, QString param)
     QStringList arguments;
 
     #if defined(Q_WS_MAEMO_5)
-	//On Maemo 5 the helper script resides in /opt/usr/bin, which is usually not in $PATH
-	arguments.append( "/opt/usr/bin/QCPUFreq.helper" );
+    //On Maemo 5 the helper script resides in /opt/usr/bin, which is usually not in $PATH
+    arguments.append( "/opt/usr/bin/QCPUFreq.helper" );
     #else
-	arguments.append( "QCPUFreq.helper" );
+    arguments.append( "QCPUFreq.helper" );
     #endif
 
     arguments.append( action );
     arguments.append( param );
 
-    helperProcess->start( "sudo", arguments, QIODevice::NotOpen );
+    helperProcess.start( "sudo", arguments, QIODevice::NotOpen );
 
-    if ( showSudoError && !helperProcess->waitForFinished( 400 )) {
-	//do not show this error again
-	showSudoError = false;
-	QMessageBox::critical(this, tr("QCPUFreq"), tr("There seems to be a problem with your sudo setup!"));
+    if ( showSudoError && !helperProcess.waitForFinished( 400 )) {
+        //do not show this error again
+        showSudoError = false;
+        QMessageBox::critical(this, tr("QCPUFreq"), tr("There seems to be a problem with your sudo setup!"));
     }
 
-    return helperProcess->exitCode();
+    return helperProcess.exitCode();
 }
 
 
@@ -161,24 +176,24 @@ QString MainWindow::getCPUTemp()
 
     //check if we can read a more accurate temperature (only for power kernel)
     if (file.exists())
-	return QString( readSysFile( "class/power_supply/bq27200-0/temp" ) + " " + QString::fromUtf8("\302\260") + "C" );
+        return QString( readSysFile( "class/power_supply/bq27200-0/temp" ) + " " + QString::fromUtf8("\302\260") + "C" );
     else {
-	/*
-	    We actually only need to read the raw temperature, but it appears that by also reading temp1_input
-	    the raw temperature (temp1_input_raw) is being updated more frequently.
-	*/
-	readSysFile( "devices/platform/omap34xx_temp/temp1_input" );
+        /*
+          We actually only need to read the raw temperature, but it appears that by also reading temp1_input
+          the raw temperature (temp1_input_raw) is being updated more frequently.
+        */
+        readSysFile( "devices/platform/omap34xx_temp/temp1_input" );
 
-	//read the current system temperature
-	QString tstring = readSysFile( "devices/platform/omap34xx_temp/temp1_input_raw" );
-	if (tstring == "0")
-	    return tr( "Unknown" );
+        //read the current system temperature
+        QString tstring = readSysFile( "devices/platform/omap34xx_temp/temp1_input_raw" );
+        if (tstring == "0")
+            return tr( "Unknown" );
 
-	//convert it to an integer and calculate the approx. temperature from the raw value
-	int tint = tstring.toInt();
-	tint = ( 0.65 * tint );
-	tstring.setNum(tint);
-	return QString( tstring + " " + QString::fromUtf8("\302\260") + "C" );
+        //convert it to an integer and calculate the approx. temperature from the raw value
+        int tint = tstring.toInt();
+        tint = ( 0.65 * tint );
+        tstring.setNum(tint);
+        return QString( tstring + " " + QString::fromUtf8("\302\260") + "C" );
     }
 #endif
     return tr( "Unknown" );
@@ -200,8 +215,32 @@ int MainWindow::getMaxFreq()
   */
 int MainWindow::getMinFreq()
 {
-    QString tmp = readSysFile( "devices/system/cpu/cpu0/cpufreq/scaling_min_freq" );
-    return tmp.toInt();
+    if (this->minFreq == 0) {
+        QString min = readSysFile( "devices/system/cpu/cpu0/cpufreq/scaling_min_freq" );
+        //check if avoid file exists (only on power kernel)
+        QFile file( "/sys/devices/system/cpu/cpu0/cpufreq/ondemand/avoid_frequencies" );
+        if (file.exists()) {
+            QString avoid = readSysFile( "devices/system/cpu/cpu0/cpufreq/ondemand/avoid_frequencies" );
+            QStringList avoidList = avoid.split( " " );
+
+            //check if min is in avoid_frequencies
+            for (int i = getScalingStep( min.toInt() ); i>0; --i) {
+                if (!avoidList.contains(min.setNum( getScalingFreq(i) ))) {
+                    this->minFreq = min.toInt();
+                    return this->minFreq;
+                }
+            }
+
+            //should not happen at all
+            this->minFreq = 125000;
+            return this->minFreq;
+        } else {
+            this->minFreq = min.toInt();
+            return this->minFreq;
+        }
+    } else {
+        return this->minFreq;
+    }
 }
 
 
@@ -295,8 +334,8 @@ QString MainWindow::readSysFile(QString sys_file)
 
     //open the file
     if ( !file.exists() || !file.open( QIODevice::ReadOnly ) ) {
-	QMessageBox::critical(this, tr("QCPUFreq"), tr("Could not get information from /sys!"));
-	return "";
+        QMessageBox::critical(this, tr("QCPUFreq"), tr("Could not get information from /sys!"));
+        return "";
     }
 
     //read the file
@@ -362,18 +401,18 @@ void MainWindow::orientationChanged()
     if ( usePortrait() ) {
 	//in portrait mode we want to display the large image
 	image.load( ":/img/chip256" );
-	this->scene->clear();
-	this->scene->addPixmap(  image  );
+	scene.clear();
+	scene.addPixmap(  image  );
 
-	ui->graphicsPortrait->setScene( this->scene );
+	ui->graphicsPortrait->setScene( &scene );
 	ui->graphicsPortrait->setMaximumSize( 256, 256 );
 	ui->graphicsLandscape->setMaximumSize( 0, 0 );
     } else {
 	image.load( ":/img/chip128" );
-	this->scene->clear();
-	this->scene->addPixmap(  image  );
+	scene.clear();
+	scene.addPixmap(  image  );
 
-	ui->graphicsLandscape->setScene( this->scene );
+	ui->graphicsLandscape->setScene( &scene );
 	ui->graphicsLandscape->setMaximumSize( 128, 128 );
 	ui->graphicsPortrait->setMaximumSize( 0, 0 );
     }
@@ -391,6 +430,22 @@ void MainWindow::setAutoRotation()
 }
 
 
+/**
+  * SLOT: enable/disable overclocking.
+  */
+void MainWindow::setOverclocking()
+{
+    if (ui->actionOverclocking->isChecked()) {
+        #if defined(Q_WS_MAEMO_5)
+        QMaemo5InformationBox::information(this, tr( "Please note that overclocking voids your warranty and may break your device! Be careful!"), 0);
+        #endif
+        this->allowOverclocking = true;
+    } else {
+        this->allowOverclocking = false;
+    }
+}
+
+
 /**
   * SLOT: Enables or disables Smart Reflex(tm) after pressing sr_btn
   */
@@ -399,10 +454,10 @@ void MainWindow::setSmartReflex()
 //SmartReflex is only supported on Maemo5
 #if defined(Q_WS_MAEMO_5)
     if ( getSmartReflexState() == 1 )
-	callHelper( "set_sr", "off");
+        callHelper( "set_sr", "off");
     else {
-	QMaemo5InformationBox::information(this, tr( "SmartReflex support is known to be unstable on some devices and may cause random reboots." ), QMaemo5InformationBox::DefaultTimeout);
-	callHelper( "set_sr", "on");
+        QMaemo5InformationBox::information(this, tr( "SmartReflex support is known to be unstable on some devices and may cause random reboots." ), QMaemo5InformationBox::DefaultTimeout);
+        callHelper( "set_sr", "on");
     }
 
 #endif
@@ -416,7 +471,7 @@ void MainWindow::setSmartReflex()
   */
 void MainWindow::showHelp()
 {
-    helpWindow->show();
+    helpWindow.show();
 }
 
 
@@ -427,7 +482,7 @@ bool MainWindow::usePortrait()
 {
     QRect screenGeometry = QApplication::desktop()->screenGeometry();
     if (screenGeometry.width() > screenGeometry.height())
-	return false;
+        return false;
     else
-	return true;
+        return true;
 }
