@@ -44,12 +44,14 @@ public slots:
     void orientationChanged();
     void refresh();
     void setAutoRotation();
+    void setOverclocking();
     void setSmartReflex();
     void showHelp();
 
 
 private:
     Ui::MainWindow *ui;
+    bool allowOverclocking;
     int callHelper( QString action, QString param );
     QString getCPUTemp();
     int getMaxFreq();
@@ -60,14 +62,15 @@ private:
     int getScalingSteps();
     int getSmartReflexState();
     //! The helper process
-    QProcess *helperProcess;
+    QProcess helperProcess;
     //! The help window
-    HelpWindow *helpWindow;
+    HelpWindow helpWindow;
+    int minFreq;
     QString readSysFile( QString sys_file );
     //! the timer for refreshing the UI
-    QTimer *refreshTimer;
+    QTimer refreshTimer;
     //! the QGraphicsScene will contain the large chip icon displayed in the UI
-    QGraphicsScene *scene;
+    QGraphicsScene scene;
     bool showSudoError;
     bool usePortrait();
 };
